-- This file is part of CyrHUD
--
-- (C) 2016 Scott Yeskie (Sasky)
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.

CyrHUD = CyrHUD or {}
CyrHUD.addonVars = {}
CyrHUD.addonVars.version	= "2024.05.18"
CyrHUD.addonVars.name 		= "CyrHUD"
CyrHUD.addonVars.author 	= "Sasky, |c4779ce@aldericon|r, |c3CB371@Masteroshi430|r"
CyrHUD.addonVars.website	= "http://www.esoui.com/downloads/fileinfo.php?id=559#info"
-- CyrHUD.yourKills = 0
-- CyrHUD.yourDeaths = 0

----------------------------------------------
-- Utility
----------------------------------------------

local function bl(val)
	if val == nil then return "NIL" elseif val then return "T" else return "F" end
end

local function nn(val)
    if val == nil then return "NIL" end
    return val
end

function CyrHUD.formatTime(delta, inclueSec, doAdditional)
    local sec = delta % 60
    delta = (delta - sec) / 60
    local min = delta % 60
    local out = min .. "m"

	if doAdditional == true then
		if min == 0 then
		    if sec == 0 then
			    out = '  '.."now"
			else
			    out = '  '..sec.."s"
			end
			
		elseif min > 9 then
            out = '  '..out 		
		else
		    if sec < 10 then sec = "0"..sec end
			out = '  '..min..":"..sec
		end
	end

    if inclueSec then
        out = out .. " " .. sec .. "s"
    end

    return out
end

CyrHUD.errors = {}

function CyrHUD:error(val)
    if not self.errors[val] then
        self.errors[val] = 1
        d("|cFF0000ERROR (CyrHUD): " .. val .. "\n|CCCCCCCPlease file this bug info with a screenshot at |CEEEEFFesoui.com (CyrHUD)")
    end
end

----------------------------------------------
-- Events
----------------------------------------------

function CyrHUD.eventAttackChange(_, keepID, battlegroundContext, underAttack)
    local self = CyrHUD

    --Optionally hide IC district battles
    if GetKeepType(keepID) == KEEPTYPE_IMPERIAL_CITY_DISTRICT then
        if CyrHUD.cfg.hideImpBattles then
            return
        end
    end
	
	if (GetKeepType(keepID) == KEEPTYPE_BRIDGE or GetKeepType(keepID) == KEEPTYPE_MILEGATE) and CyrHUD.cfg.hideBridgesAndMilegates then
	   return
	end
	
	-- We ignore new resource and gate/bridge info when there is more than 10 entries to avoid too much infos
	if (GetKeepType(keepID) == KEEPTYPE_BRIDGE or GetKeepType(keepID) == KEEPTYPE_MILEGATE or GetKeepType(keepID) == KEEPTYPE_RESOURCE) and not self.battles[keepID] and CyrHUD.entryCount > 10 then
	    return
	end

	
	if GetKeepType(keepID) == KEEPTYPE_ARTIFACT_GATE or GetKeepType(keepID) == KEEPTYPE_BRIDGE or GetKeepType(keepID) == KEEPTYPE_MILEGATE then
	
		local pinType,_,_ = GetKeepPinInfo(keepID, battlegroundContext)
		local gateOpen = false
		if pinType == MAP_PIN_TYPE_ARTIFACT_GATE_OPEN_ALDMERI_DOMINION or pinType == MAP_PIN_TYPE_ARTIFACT_GATE_OPEN_DAGGERFALL_COVENANT or pinType == MAP_PIN_TYPE_ARTIFACT_GATE_OPEN_EBONHEART_PACT then
			 gateOpen = true
		elseif pinType == MAP_PIN_TYPE_KEEP_BRIDGE_IMPASSABLE or pinType == MAP_PIN_TYPE_KEEP_MILEGATE_IMPASSABLE or pinType == MAP_PIN_TYPE_KEEP_MILEGATE_CENTER_DESTROYED then
		    gateOpen = true -- yep, it should be keepImpassable = true but the code is cleaner/has less lines like that   
		end
	end


    if underAttack or gateOpen then
        self:add(keepID)
    elseif self.battles[keepID] ~= nil then
        self.battles[keepID]:update()
    end

    self.battleContext = battlegroundContext
end


function CyrHUD.SetMovingObjective(_, keepId, objectiveId, battlegroundContext, objectiveName, objectiveType, objectiveControlEvent, state, holdingAlliance, attackingAlliance, pinType)
   local self = CyrHUD

    

    if state == OBJECTIVE_CONTROL_STATE_FLAG_DROPPED or state == OBJECTIVE_CONTROL_STATE_FLAG_HELD or objectiveControlEvent == OBJECTIVE_CONTROL_EVENT_FLAG_SPAWNED then
        objectiveName = LocalizeString('<<1>>', objectiveName)
		
        CyrHUD.MovingObjective[objectiveId] = CyrHUD.MovingObjective[objectiveId] or {}
		CyrHUD.MovingObjective[objectiveId].startMovingObjective = CyrHUD.MovingObjective[objectiveId].startMovingObjective or GetTimeStamp()
		CyrHUD.MovingObjective[objectiveId].endMovingObjective = nil
		local _, holderName = GetCarryableObjectiveHoldingCharacterInfo(keepId, objectiveId, battlegroundContext)
		CyrHUD.MovingObjective[objectiveId].holder = holderName
        CyrHUD.MovingObjective[objectiveId].name = objectiveName
		CyrHUD.MovingObjective[objectiveId].objectiveId = objectiveId 
		CyrHUD.MovingObjective[objectiveId].keepId = keepId
		CyrHUD.MovingObjective[objectiveId].objectiveType = objectiveType
		CyrHUD.MovingObjective[objectiveId].objectiveControlEvent = objectiveControlEvent
		CyrHUD.MovingObjective[objectiveId].holdingAlliance = holdingAlliance
		CyrHUD.MovingObjective[objectiveId].color = CyrHUD.info[holdingAlliance].color:UnpackRGBA()
		CyrHUD.MovingObjective[objectiveId].texture = ZO_MapPin.PIN_DATA[pinType].texture
		CyrHUD.MovingObjective[objectiveId].type = "MovingObjective"


	   if self.MovingObjectives[objectiveId] ~= nil then
	       CyrHUD.MovingObjectives[objectiveId]:update()
	   else
	       self:addObj(objectiveId)
	   end

   else
       -- remove the objective if not held or dropped
	   if CyrHUD.MovingObjectives[objectiveId] then CyrHUD.MovingObjectives[objectiveId] = nil end 
   end


   self.battleContext = battlegroundContext 
end

local g_pvpKillFeedDeathRecurrenceTracker = nil
do
    -- The PvP Kill Feed uses ZO_RecurrenceTracker to track whether any given
    -- killer/victim message has been shown within the last 10 seconds from a
    -- given source (local vs. kill location). Note that the instance count
    -- tracked by ZO_RecurrenceTracker is irrelevant here for the purpose of
    -- the kill feed.
    local EXPIRATION_MS = 10000 -- 10 seconds
    local EXTENSION_MS = 10000 -- 10 seconds
    g_pvpKillFeedDeathRecurrenceTracker = ZO_RecurrenceTracker:New(EXPIRATION_MS, EXTENSION_MS)
end

function CyrHUD.playerKilled(_, killLocation, killerPlayerDisplayName, killerPlayerCharacterName, killerPlayerAlliance, killerPlayerRank, victimPlayerDisplayName, victimPlayerCharacterName, victimPlayerAlliance, victimPlayerRank)
    local self = CyrHUD
	
	killLocation = LocalizeString('<<1>>', killLocation)
	
	-- clean previous data if no deaths in the place during the last 6 mn
	if CyrHUD.Graveyard and CyrHUD.Graveyard[killLocation] and GetDiffBetweenTimeStamps(GetTimeStamp(), CyrHUD.Graveyard[killLocation].lastUpdate) > 360 then
        CyrHUD.Graveyard[killLocation] = nil
    end	

	
	-- ZOS' spam filter
	local messageKeySuffix = string.format("%s___%s", killerPlayerDisplayName, victimPlayerDisplayName)
	local messageKeyLocal = "L" .. messageKeySuffix
	local messageKeyKillLocation = "B" .. messageKeySuffix
	if isKillLocation then
		-- This message was kill location sourced.
		if g_pvpKillFeedDeathRecurrenceTracker:RemoveValue(messageKeyLocal) ~= nil then
			-- The same message was already shown as a result of a local message;
			-- remove the original message from the tracker and suppress this message.
			return
		end
		-- Track this kill location sourced message.
		g_pvpKillFeedDeathRecurrenceTracker:AddValue(messageKeyKillLocation)
	else
		-- This message was locally sourced.
		if g_pvpKillFeedDeathRecurrenceTracker:RemoveValue(messageKeyKillLocation) ~= nil then
			-- The same message was already shown as a result of a kill location message;
			-- remove the original message from the tracker and suppress this message.
			return
		end
		-- Track this locally sourced message.
		g_pvpKillFeedDeathRecurrenceTracker:AddValue(messageKeyLocal)
	end

	
	-- populate your kills and deaths
	-- local yourDisplayName = GetDisplayName()
	-- if killerPlayerDisplayName == yourDisplayName then
	       -- CyrHUD.yourKills = CyrHUD.yourKills + 1
	-- elseif victimPlayerDisplayName == yourDisplayName then
	       -- CyrHUD.yourDeaths = CyrHUD.yourDeaths + 1
	-- end
	
	-- We terminate previous graveyards in case there is a new one when there is more than 10 entries to avoid too much infos 
	if not self.Graveyards[killLocation] and CyrHUD.entryCount > 10 then
	   for k, _ in pairs(self.Graveyards) do
           self.Graveyards[k].endGraveyard = GetTimeStamp()
		   CyrHUD.Graveyards[k] = nil
       end
	end
	
	CyrHUD.Graveyard = CyrHUD.Graveyard or {}
	CyrHUD.Graveyard[killLocation] = CyrHUD.Graveyard[killLocation] or {}
    CyrHUD.Graveyard[killLocation].startGraveyard = CyrHUD.Graveyard[killLocation].startGraveyard or GetTimeStamp()
	CyrHUD.Graveyard[killLocation].name = killLocation
	CyrHUD.Graveyard[killLocation].endGraveyard = nil
	CyrHUD.Graveyard[killLocation].lastUpdate = GetTimeStamp()
	CyrHUD.Graveyard[killLocation].type = "Graveyard"
	
	-- avoid nil values
	CyrHUD.Graveyard[killLocation].allianceKills = CyrHUD.Graveyard[killLocation].allianceKills or {}
	CyrHUD.Graveyard[killLocation].allianceKills[1] = CyrHUD.Graveyard[killLocation].allianceKills[1] or 0
	CyrHUD.Graveyard[killLocation].allianceKills[2] = CyrHUD.Graveyard[killLocation].allianceKills[2] or 0
	CyrHUD.Graveyard[killLocation].allianceKills[3] = CyrHUD.Graveyard[killLocation].allianceKills[3] or 0
	CyrHUD.Graveyard[killLocation].allianceDeaths = CyrHUD.Graveyard[killLocation].allianceDeaths or {}
	CyrHUD.Graveyard[killLocation].allianceDeaths[1] = CyrHUD.Graveyard[killLocation].allianceDeaths[1] or 0
	CyrHUD.Graveyard[killLocation].allianceDeaths[2] = CyrHUD.Graveyard[killLocation].allianceDeaths[2] or 0
	CyrHUD.Graveyard[killLocation].allianceDeaths[3] = CyrHUD.Graveyard[killLocation].allianceDeaths[3] or 0
	
	-- increment the location's alliance counters for kills & deaths
	CyrHUD.Graveyard[killLocation].allianceKills[killerPlayerAlliance] = CyrHUD.Graveyard[killLocation].allianceKills[killerPlayerAlliance] + 1
	CyrHUD.Graveyard[killLocation].allianceDeaths[victimPlayerAlliance] = CyrHUD.Graveyard[killLocation].allianceDeaths[victimPlayerAlliance] + 1
	
	-- calculate if we are in the winning alliance
	local ADscore = CyrHUD.Graveyard[killLocation].allianceKills[1] - CyrHUD.Graveyard[killLocation].allianceDeaths[1]
	local DCscore = CyrHUD.Graveyard[killLocation].allianceKills[3] - CyrHUD.Graveyard[killLocation].allianceDeaths[3]
	local EPscore = CyrHUD.Graveyard[killLocation].allianceKills[2] - CyrHUD.Graveyard[killLocation].allianceDeaths[2]
	
	--d("ADscore: "..ADscore.." EPscore: "..EPscore.." DCscore: "..DCscore)

	if ADscore >= DCscore and ADscore >= EPscore then
	       CyrHUD.Graveyard[killLocation].winningAlliance = 1
	elseif DCscore >= ADscore and DCscore >= EPscore then
	       CyrHUD.Graveyard[killLocation].winningAlliance = 3
	elseif EPscore >= ADscore and EPscore >= DCscore then  
	       CyrHUD.Graveyard[killLocation].winningAlliance = 2
	else
	       CyrHUD.Graveyard[killLocation].winningAlliance = 0
	end
	
	
	-- the data is entered but has it enough deaths (10) to be displayed? 
	if CyrHUD.Graveyard[killLocation].allianceDeaths[1] + CyrHUD.Graveyard[killLocation].allianceDeaths[2] + CyrHUD.Graveyard[killLocation].allianceDeaths[3] < 10 then
	   return
	end
	
	
	-- choose the right texture 
	local ADinvolved = (CyrHUD.Graveyard[killLocation].allianceKills[1] + CyrHUD.Graveyard[killLocation].allianceDeaths[1]) > 0 
	local DCinvolved = (CyrHUD.Graveyard[killLocation].allianceKills[3] + CyrHUD.Graveyard[killLocation].allianceDeaths[3]) > 0
	local EPinvolved = (CyrHUD.Graveyard[killLocation].allianceKills[2] + CyrHUD.Graveyard[killLocation].allianceDeaths[2]) > 0
	
	if ADinvolved and DCinvolved and EPinvolved then
	      CyrHUD.Graveyard[killLocation].texture = "EsoUI/Art/MapPins/AvA_3Way.dds"
	elseif ADinvolved and DCinvolved then
	       CyrHUD.Graveyard[killLocation].texture = "EsoUI/Art/MapPins/AvA_AldmeriVDaggerfall.dds"
	elseif ADinvolved and EPinvolved then
	       CyrHUD.Graveyard[killLocation].texture = "EsoUI/Art/MapPins/AvA_AldmeriVEbonheart.dds"
	elseif DCinvolved and EPinvolved then
	       CyrHUD.Graveyard[killLocation].texture = "EsoUI/Art/MapPins/AvA_EbonheartVDaggerfall.dds"
	else
	       CyrHUD.Graveyard[killLocation].texture = "/esoui/art/treeicons/gamepad/gp_tutorial_idexicon_death.dds"
	end
	
	if self.Graveyards[killLocation] ~= nil then
	    CyrHUD.Graveyards[killLocation]:update()
	else
	    self:addGraveyard(killLocation)
	end

end


function CyrHUD.onMonsterDeath(_, unitTag, isDead)
    if GetCurrentMapId() ~= 660 then return end -- only in imperial city upper district
	-- mapid 785 is Barathrum Centrata (Molag Bal) 
    local self = CyrHUD	
	local monsterName = GetUnitName(unitTag)
	local isDeadly = GetUnitDifficulty(unitTag) == MONSTER_DIFFICULTY_DEADLY
	
	if monsterName == nil or monsterName == "" or not isDeadly then return end -- Abort if unit is not part of Patrolling Horrors

	if isDead == true then -- Imperial City Boss just died
	     
		local currentAreaName = string.gsub(GetPlayerLocationName(), "(%w+)[%^]+.*", "%1") 
		if not currentAreaName then return end
		local areaBossName = currentAreaName.." "..GetString(SI_CUSTOMERSERVICESUBMITFEEDBACKSUBCATEGORIES501)
		
		if self.PatrollingHorrors[areaBossName] ~= nil then
			CyrHUD.PatrollingHorrors[areaBossName]:restart()
		else
			self:addPatrollingHorror(areaBossName)
		end

	end
end

function CyrHUD.onMonsterReticle(_)
    if GetCurrentMapId() ~= 660 then return end -- only in imperial city upper district
    local self = CyrHUD
	local unitName = GetUnitNameHighlightedByReticle()
    local isDeadly = GetUnitDifficulty('reticleover') == MONSTER_DIFFICULTY_DEADLY
		
	if unitName == nil or unitName == "" or not isDeadly then return end -- Abort if unit is not part of Patrolling Horrors
	
	local isDead = IsUnitDead('reticleover')
    local currentAreaName = string.gsub(GetPlayerLocationName(), "(%w+)[%^]+.*", "%1")
	if not currentAreaName then return end
	local areaBossName = currentAreaName.." "..GetString(SI_CUSTOMERSERVICESUBMITFEEDBACKSUBCATEGORIES501) 
	
	if isDead == true then -- Imperial City Boss is dead on the floor 
		if self.PatrollingHorrors[areaBossName] then
			if self.PatrollingHorrors[areaBossName].lastSeenAlive and GetDiffBetweenTimeStamps( GetTimeStamp(), self.PatrollingHorrors[areaBossName].lastSeenAlive) < 5 then -- died before your eyes
			       self.PatrollingHorrors[areaBossName].lastSeenAlive = nil
                   CyrHUD.PatrollingHorrors[areaBossName]:restart()  			
			elseif self.PatrollingHorrors[areaBossName]:getRawDuration() > 960 then -- Already dead before you arrive, we end the notification
				self.PatrollingHorrors[areaBossName].endPatrollingHorror = GetTimeStamp()
			end
        end

	else -- Imperial City Boss is facing you alive  
	    if self.PatrollingHorrors[areaBossName] == nil then -- no notification yet 
            self:engagePatrollingHorror(areaBossName)  
        elseif (900 - self.PatrollingHorrors[areaBossName]:getRawDuration()) > 0 then -- adjust time
		     self.PatrollingHorrors[areaBossName].startPatrollingHorror = GetTimeStamp() - 900
		else
             self.PatrollingHorrors[areaBossName].lastSeenAlive = GetTimeStamp()		
		end	
         		
	end
end




function CyrHUD.saveWindowPosition( window )
    local _, sP, _, aP, x, y = window:GetAnchor()
    CyrHUD.cfg.anchPoint = aP
    CyrHUD.cfg.selfPoint = sP
    CyrHUD.cfg.xoff = x
    CyrHUD.cfg.yoff = y
end

function CyrHUD.actionLayerChange(_, _, activeLayerIndex)
    CyrHUD_UI:SetHidden(activeLayerIndex > 2)
end

----------------------------------------------
-- Notification UI pool
----------------------------------------------

CyrHUD.entryCount = 0
CyrHUD.entries = {}

function CyrHUD:reconfigureLabels()
    for _,entry in pairs(self.entries) do
        --Forces reconfigure on next update
        entry.type = nil
    end
end

function CyrHUD:hideRow(index)
    if self.entries[index] then
        self.entries[index].main:SetHidden(true)
    end
end

function CyrHUD:getUIRow(index)
    if #self.entries < index then
        table.insert(self.entries, self.Label())
        index = #self.entries
    end

    self.entries[index].main:SetHidden(false)

    return self.entries[index]
end

function CyrHUD:printAll()
	local i = 1

    for _,status in ipairs(self.statusBars) do
        self:getUIRow(i):update(status)
        i = i + 1
    end

    for _,battle in pairs(self.battles) do
        self:getUIRow(i):update(battle)
        i = i + 1
    end
	
	for _,MovingObjective in pairs(self.MovingObjectives) do
        self:getUIRow(i):update(MovingObjective)
        i = i + 1
    end
	
	for _,Graveyard in pairs(self.Graveyards) do
        self:getUIRow(i):update(Graveyard)
        i = i + 1
    end
	
	for _,PatrollingHorror in pairs(self.PatrollingHorrors) do
        self:getUIRow(i):update(PatrollingHorror)
        i = i + 1
    end

    --Fix since auto-resize doesn't seem to work well
    self.ui:SetHeight(math.max(i*35,70))
	
	CyrHUD.entryCount = i-2


    for j=i,#self.entries do
        self:hideRow(j)
    end

end

----------------------------------------------
-- Battle management
----------------------------------------------

CyrHUD.battles = {}
CyrHUD.MovingObjectives = {}
CyrHUD.Graveyards = {} 
CyrHUD.PatrollingHorrors = {} 

function CyrHUD:add(keepID)
    if self.battles[keepID] == nil then
        self.battles[keepID] = self.Battle(keepID)
    else
        self.battles[keepID]:restart()
    end
end

function CyrHUD:checkAdd(keepID)
    if self.battles[keepID] == nil then
        local battle = self.Battle(keepID)

        if battle:isBattle() then 
            self.battles[keepID] = battle
		end
    elseif self.battles[keepID]:isBattle() then
        self.battles[keepID]:restart()
    end
end

function CyrHUD:scanKeeps()

    if IsInImperialCity() then
	    -- Districts
		self:checkAdd(141)
		self:checkAdd(142)
		self:checkAdd(143)
		self:checkAdd(146)
		self:checkAdd(147)
		self:checkAdd(148)

	elseif IsInCyrodiil() then
	    -- Keeps / Resources
		for i=3,87 do
			self:checkAdd(i)
		end

		-- Outposts
		for i=132,134 do
			self:checkAdd(i)
		end
        
		-- Outposts
		for i=163,165 do
			self:checkAdd(i)
		end

		-- Towns
		self:checkAdd(149)
		self:checkAdd(151)
		self:checkAdd(152)

        if not CyrHUD.cfg.hideBridgesAndMilegates then
			-- Bridges / Milegates
			for i=154,162 do 
				self:checkAdd(i)
			end
		end
		
		-- Scroll temple Gates
		for i=124,129 do 
			self:checkAdd(i)
		end	
		
		-- Border Keeps: 105 to 110
		
		-- Scroll temples: 118 to 123
		   -- 118 altadoon 124 it's gate
		   -- 119 mnem     125 it's gate
		   -- 120 ghartok  126 it's gate
		   -- 121 chim     127 it's gate
		   -- 122 ni mohk  128 it's gate
		   -- 123 alma ruma 129 it's gate

		
		-- held Volendrung & Scrolls
		for i = 1, GetNumObjectives() do
		   local okeepId, objectiveId, obgContext = GetAvAObjectiveKeysByIndex(i)
	        if(IsLocalBattlegroundContext(obgContext)) then
			    local objectiveName, objectiveType, objectiveState = GetObjectiveInfo(okeepId, objectiveId, obgContext)
			   if objectiveType == OBJECTIVE_ARTIFACT_DEFENSIVE or objectiveType == OBJECTIVE_ARTIFACT_OFFENSIVE or objectiveType == OBJECTIVE_DAEDRIC_WEAPON then
			      local objectiveControlEvent = GetLastObjectiveControlEvent(okeepId, objectiveId, obgContext)
			      if objectiveState == OBJECTIVE_CONTROL_STATE_FLAG_DROPPED or objectiveState == OBJECTIVE_CONTROL_STATE_FLAG_HELD or objectiveControlEvent == OBJECTIVE_CONTROL_EVENT_FLAG_SPAWNED then
			         local holdingAlliance, lastHoldingAlliance = GetCarryableObjectiveHoldingAllianceInfo(okeepId, objectiveId, obgContext)
				     local pinType = GetObjectivePinInfo(okeepId, objectiveId, obgContext) 
			         CyrHUD.SetMovingObjective(nil, okeepId, objectiveId, obgContext, objectiveName, objectiveType, objectiveControlEvent, objectiveState, holdingAlliance, lastHoldingAlliance, pinType)
			       
			      end 
			   end
		    end 
        end		

	end
end

----------------------------------------------
-- held Scrolls & Volendrung management
----------------------------------------------


function CyrHUD:addObj(objectiveId)
    if self.MovingObjectives[objectiveId] == nil then
        self.MovingObjectives[objectiveId] = self.MovingObjective(objectiveId)
    else
        self.MovingObjectives[objectiveId]:restart()
    end
end

function CyrHUD:checkAddObj(objectiveId)
    if self.MovingObjectives[objectiveId] == nil then
        local MovingObjective = self.MovingObjective(objectiveId)
       
        self.MovingObjectives[objectiveId] = MovingObjective
		
    else
        self.MovingObjectives[objectiveId]:restart()
    end
end


----------------------------------------------
-- Graveyards management
----------------------------------------------


function CyrHUD:addGraveyard(killLocation)
    if self.Graveyards[killLocation] == nil then
         self.Graveyards[killLocation] = self.Graveyard(killLocation)
    else
         self.Graveyards[killLocation]:restart()
    end
end

function CyrHUD:checkAddGraveyard(killLocation)
    if self.Graveyards[killLocation] == nil then
        local Graveyard = self.Graveyard[killLocation]
       
        self.Graveyards[killLocation] = Graveyard
		
    else
        self.Graveyards[killLocation]:restart()
    end
end

----------------------------------------------
-- Patrolling Horrors management
----------------------------------------------


function CyrHUD:addPatrollingHorror(areaBossName)
    if self.PatrollingHorrors[areaBossName] == nil then
         self.PatrollingHorrors[areaBossName] = self.PatrollingHorror(areaBossName)
    else
         self.PatrollingHorror[areaBossName]:restart()
    end
end

function CyrHUD:engagePatrollingHorror(areaBossName)
    if self.PatrollingHorrors[areaBossName] == nil then
         self.PatrollingHorrors[areaBossName] = self.PatrollingHorror(areaBossName, true)
    end
end

function CyrHUD:checkAddPatrollingHorror(areaBossName)
    if self.PatrollingHorrors[areaBossName] == nil then
        local PatrollingHorror = self.PatrollingHorror[areaBossName]
       
        self.PatrollingHorrors[areaBossName] = PatrollingHorror
		
    else
        self.PatrollingHorrors[areaBossName]:restart()
    end
end

--------------------------------------------------


function CyrHUD:updateAll()
    for i,_ in pairs(self.battles) do
        --Update in-place
        self.battles[i]:update()
    end
	
	for j,_ in pairs(self.MovingObjectives) do
       self.MovingObjectives[j]:update()
    end
	
	for k, _ in pairs(self.Graveyards) do
       self.Graveyards[k]:update()
    end
	
	for k, _ in pairs(self.PatrollingHorrors) do
       self.PatrollingHorrors[k]:update()
    end

    for _,status in pairs(self.statusBars) do
        status:update()
    end
end



------------------------------------------------------------------------
-- Initialization
------------------------------------------------------------------------

CyrHUD.visible = false

function CyrHUD:init()
    --Init UI
    self:disableQuestTrackers()
    self.ui:SetHidden(false)

    --Populate data
    self:refresh()

    --Add events
    EVENT_MANAGER:RegisterForUpdate(CyrHUD.addonVars.name .. "KeepCheck", 5000, function()
        self:scanKeeps()
        self:updateAll()
		
		-- for testing scrolls & volendrung only (generate fake ones)
		-- if not trumpet then   
		     -- CyrHUD.SetMovingObjective(_, 2000, 2000, CyrHUD.battleContext, "Elder Scroll of Alma Ruma", OBJECTIVE_DAEDRIC_WEAPON, 145, OBJECTIVE_CONTROL_STATE_FLAG_DROPPED, 1, 2, MAP_PIN_TYPE_AVA_DAEDRIC_ARTIFACT_VOLENDRUNG_ALDMERI)
		     -- trumpet = true
		-- else
		     -- CyrHUD.SetMovingObjective(_, 2000, 2000, CyrHUD.battleContext, "Elder Scroll of Alma Ruma", OBJECTIVE_DAEDRIC_WEAPON, 145, OBJECTIVE_CONTROL_STATE_FLAG_AT_ENEMY_BASE, 1, 2, MAP_PIN_TYPE_AVA_DAEDRIC_ARTIFACT_VOLENDRUNG_ALDMERI)
		     -- trumpet = false  
		-- end
	
		
		--for testing Graveyards only (generate fake ones)
		-- local killLocation = GetKeepName(math.random(162))
		-- if not killLocation or killLocation == "" then return end
		-- local killerPlayerAlliance = math.random(3)
		-- local victimPlayerAlliance = killerPlayerAlliance
		-- while (victimPlayerAlliance == killerPlayerAlliance) do
		       -- victimPlayerAlliance = math.random(3)
		-- end
		-- if not saxophone then 
		
		     -- CyrHUD.playerKilled(_, killLocation, killerPlayerAlliance, victimPlayerAlliance)
		     -- saxophone = true
		-- else
		     -- CyrHUD.playerKilled(_, killLocation, killerPlayerAlliance, victimPlayerAlliance)
		     -- saxophone = false  
		-- end
		
		
		
		--for testing Patrolling Horrors only (generate fake ones)
		-- if not piano then 
		     -- local self = CyrHUD	
			 -- local currentAreaName = string.gsub(GetPlayerLocationName(), "(%w+)[%^]+.*", "%1")
			 -- if not currentAreaName then return end
			 -- local areaBossName = currentAreaName.." "..GetString(SI_CUSTOMERSERVICESUBMITFEEDBACKSUBCATEGORIES501)
			
			 -- if self.PatrollingHorrors[areaBossName] ~= nil then
			 
				-- CyrHUD.PatrollingHorrors[areaBossName]:restart()
			 -- else
			  	-- self:addPatrollingHorror(areaBossName)
			 -- end
		     -- piano = true
		-- else
		     -- local self = CyrHUD
                -- local currentAreaName = string.gsub(GetPlayerLocationName(), "(%w+)[%^]+.*", "%1")
				-- if not currentAreaName then return end
				-- local areaBossName = currentAreaName.." "..GetString(SI_CUSTOMERSERVICESUBMITFEEDBACKSUBCATEGORIES501)
				
				-- if self.PatrollingHorrors[areaBossName] and self.PatrollingHorrors[areaBossName]:getRawDuration() >= 60 then -- Already dead before you arrive, we end the notification
					-- self.PatrollingHorrors[areaBossName].endPatrollingHorror = GetTimeStamp()
				-- end
		     -- piano = false  
		-- end
		
		
    end)

    EVENT_MANAGER:RegisterForUpdate(CyrHUD.addonVars.name .. "UIUpdate", 1000, function()
        self:printAll()
    end)

    EVENT_MANAGER:RegisterForEvent(CyrHUD.addonVars.name.."AttackChange", EVENT_KEEP_UNDER_ATTACK_CHANGED, self.eventAttackChange)
	EVENT_MANAGER:RegisterForEvent(CyrHUD.addonVars.name.."ObjectiveControlState", EVENT_OBJECTIVE_CONTROL_STATE, self.SetFlagStateData)
	EVENT_MANAGER:RegisterForEvent(CyrHUD.addonVars.name.."ArtifactControlState", EVENT_ARTIFACT_CONTROL_STATE, self.SetArtifactStateData)
	EVENT_MANAGER:RegisterForEvent(CyrHUD.addonVars.name.."GateChange", EVENT_KEEP_GATE_STATE_CHANGED, function(_, keepID, open) self.eventAttackChange(_, keepID, CyrHUD.battleContext)  end)
    EVENT_MANAGER:RegisterForEvent(CyrHUD.addonVars.name.."PassableChange", EVENT_KEEP_IS_PASSABLE_CHANGED, function(_, keepId, battlegroundContext, isPassable) self.eventAttackChange(_, keepID, CyrHUD.battleContext) end)
	EVENT_MANAGER:RegisterForEvent(CyrHUD.addonVars.name.."KillFeed", EVENT_PVP_KILL_FEED_DEATH, function(_, killLocation, killerPlayerCharacterName, killerPlayerDisplayName, killerPlayerAlliance, killerPlayerRank, victimPlayerCharacterName, victimPlayerDisplayName, victimPlayerAlliance, victimPlayerRank)  CyrHUD.playerKilled(_, killLocation, killerPlayerCharacterName, killerPlayerDisplayName, killerPlayerAlliance, killerPlayerRank, victimPlayerCharacterName, victimPlayerDisplayName, victimPlayerAlliance, victimPlayerRank) end)

    self.visible = true

    EVENT_MANAGER:RegisterForEvent(CyrHUD.addonVars.name, EVENT_ACTION_LAYER_POPPED, self.actionLayerChange)
    EVENT_MANAGER:RegisterForEvent(CyrHUD.addonVars.name, EVENT_ACTION_LAYER_PUSHED, self.actionLayerChange)
end

function CyrHUD:refresh()
    --Get initial scan
    self.battles = {}
	self.MovingObjectives = {}
	self.Graveyards = {}
	self.PatrollingHorrors  = {}
    self.battleContext = BGQUERY_LOCAL
	
	if self.campaign ~= GetCurrentCampaignId() then 
	    self:disableQuestTrackers()
	    CyrHUD.imperialKeeps = nil -- reset imperial keeps/ Imperial City Districts
		
		if IsInImperialCity() and not CyrHUD.cfg.hidePatrollingHorrors then -- reset Imperial City Boss timers & manage corresponding events
		    EVENT_MANAGER:RegisterForEvent(CyrHUD.addonVars.name, EVENT_UNIT_DEATH_STATE_CHANGED, self.onMonsterDeath)
			EVENT_MANAGER:RegisterForEvent(CyrHUD.addonVars.name, EVENT_RETICLE_TARGET_CHANGED, self.onMonsterReticle) 
		else
		    EVENT_MANAGER:UnregisterForEvent(CyrHUD.addonVars.name, EVENT_UNIT_DEATH_STATE_CHANGED)
		    EVENT_MANAGER:UnregisterForEvent(CyrHUD.addonVars.name, EVENT_RETICLE_TARGET_CHANGED)
		end
		--CyrHUD.yourKills = 0
        --CyrHUD.yourDeaths = 0
		self.ImperialCityBossTimers = {}
		self.ArtifactHolders = {}
	end
	
    self.campaign = GetCurrentCampaignId()

    --Could separate this with a data refresh eventually, but just do a hard reset for now
    self.statusBars = {}
    table.insert(self.statusBars, self.ScoringBar())
    self:scanKeeps()

    --Force update on status bar
    self:reconfigureLabels()
end


function CyrHUD:setWaypoint(x,y)
    if x ~= nil and y ~= nil then
	
	   if IsUnitGroupLeader("player") then
	       --if GetCurrentMapId() ~= 16 then SetMapToMapId(16) end
	       PingMap(MAP_PIN_TYPE_RALLY_POINT, MAP_TYPE_LOCATION_CENTERED, x, y)
		   --SetMapToPlayerLocation()
	   else
	       --if GetCurrentMapId() ~= 16 then SetMapToMapId(16) end
	       PingMap(MAP_PIN_TYPE_PLAYER_WAYPOINT, MAP_TYPE_LOCATION_CENTERED, x, y)
		   --SetMapToPlayerLocation()
	   end
    end	
end

function CyrHUD:disableQuestTrackers()
    self.trackers = self.trackers or {}

    if IsInCyrodiil() then
		--Ravalox Questtracker
		if QuestTrackerWin then
			self.trackers.QuestTrackerWin = self.trackers.QuestTrackerWin or QuestTrackerWin:IsHidden() -- Tracker hidden state before CyrHUD init to restore on CyrHUD deinit  
			QuestTrackerWin:SetHidden(self.cfg.ravTrackerDisableCyro)
		end

		--ZOs build in game quest tracker
		self.trackers.ZO_FocusedQuestTrackerPanel = self.trackers.ZO_FocusedQuestTrackerPanel or ZO_FocusedQuestTrackerPanel:IsHidden() -- Tracker hidden state before CyrHUD init to restore on CyrHUD deinit  
		ZO_FocusedQuestTrackerPanel:SetHidden(self.cfg.zosTrackerDisableCyro)
		
	elseif IsInImperialCity() then
		--Ravalox Questtracker
		if QuestTrackerWin then
			self.trackers.QuestTrackerWin = self.trackers.QuestTrackerWin or QuestTrackerWin:IsHidden() -- Tracker hidden state before CyrHUD init to restore on CyrHUD deinit  
			QuestTrackerWin:SetHidden(self.cfg.ravTrackerDisableIC)
		end

		--ZOs build in game quest tracker
		self.trackers.ZO_FocusedQuestTrackerPanel = self.trackers.ZO_FocusedQuestTrackerPanel or ZO_FocusedQuestTrackerPanel:IsHidden() -- Tracker hidden state before CyrHUD init to restore on CyrHUD deinit  
		ZO_FocusedQuestTrackerPanel:SetHidden(self.cfg.zosTrackerDisableIC)
	end
end

function CyrHUD:reEnableQuestTrackers()
    if self.trackers then
        for k,v in pairs(self.trackers) do
            if _G[k] ~= nil then _G[k]:SetHidden(false) end
        end
    end
end

function CyrHUD:deinit()
    self:reEnableQuestTrackers()

    EVENT_MANAGER:UnregisterForUpdate(CyrHUD.addonVars.name.."KeepCheck")
    EVENT_MANAGER:UnregisterForUpdate(CyrHUD.addonVars.name.."UIUpdate")
    EVENT_MANAGER:UnregisterForUpdate(CyrHUD.addonVars.name.."UpdateAPCount")
    EVENT_MANAGER:UnregisterForEvent(CyrHUD.addonVars.name, EVENT_ACTION_LAYER_POPPED)
	
    EVENT_MANAGER:UnregisterForEvent(CyrHUD.addonVars.name, EVENT_ACTION_LAYER_PUSHED)
    EVENT_MANAGER:UnregisterForEvent(CyrHUD.addonVars.name.."AttackChange", EVENT_KEEP_UNDER_ATTACK_CHANGED)
    EVENT_MANAGER:UnregisterForEvent(CyrHUD.addonVars.name.."ObjectiveControlState", EVENT_OBJECTIVE_CONTROL_STATE)
	 EVENT_MANAGER:UnregisterForEvent(CyrHUD.addonVars.name.."ArtifactControlState", EVENT_ARTIFACT_CONTROL_STATE) 
	EVENT_MANAGER:UnregisterForEvent(CyrHUD.addonVars.name.."GateChange", EVENT_KEEP_GATE_STATE_CHANGED)
	EVENT_MANAGER:UnregisterForEvent(CyrHUD.addonVars.name.."PassableChange", EVENT_KEEP_IS_PASSABLE_CHANGED)
    EVENT_MANAGER:UnregisterForEvent(CyrHUD.addonVars.name.."KillFeed", EVENT_PVP_KILL_FEED_DEATH)

    CyrHUD_UI:SetHidden(true)
    self.visible = false
end

function CyrHUD.toggle()
    local self = CyrHUD

    if self.visible then
        self:deinit()
    else
        self:init()
    end
end


SLASH_COMMANDS["/cyrhud"] = CyrHUD.toggle

--Called once. Handles controls, etc.
function CyrHUD.addonInit()
    local self = CyrHUD

    --Init saved variables
    local def = {
        xoff = -10,
        yoff = 60,
        ravTrackerDisableCyro = false,
		zosTrackerDisableCyro = false,
		ravTrackerDisableIC = false,
		zosTrackerDisableIC = false,
		hideImpBattles = false,
		hideBridgesAndMilegates = false,
		hidePatrollingHorrors = false,
		enableInCyro = true,
		enableInIC = true,
        showPopBars = false,
    }

    self.cfg = ZO_SavedVars:NewAccountWide("CyrHUD_SavedVars", 1.0, "config", def)

    --Create UI
    self.ui = WINDOW_MANAGER:CreateTopLevelWindow("CyrHUD_UI")
    self.ui:SetWidth(CyrHUD.width)
    self.ui:SetMouseEnabled(true)
    self.ui:SetMovable(true)
    self.ui:SetClampedToScreen(true)
    self.ui:SetHandler("OnMoveStop", self.saveWindowPosition)

    local _, pt, relTo, relPt = CyrHUD_UI:GetAnchor()
    self.ui:ClearAnchors()
    self.ui:SetAnchor(CyrHUD.cfg.selfPoint or TOPLEFT,
        GuiRoot, CyrHUD.cfg.anchPoint or TOPRIGHT,
        CyrHUD.cfg.xoff, CyrHUD.cfg.yoff)

    --Create settings menu
    local LAM = LibAddonMenu2
    LAM:RegisterAddonPanel(CyrHUD.addonVars.name .. "-LAM", self.menuPanel)
    LAM:RegisterOptionControls(CyrHUD.addonVars.name .. "-LAM", self.menuOptions)
    self.initLAM = true

    --[[if (GetDate() % 1000)== 401 then
        --NOTE: If you see this before 4/1, please don't share
        table.insert(self.menuOptions,{
            type = "checkbox",
            name = GetString(SI_CYRHUD_APRIL1),
            tooltip = GetString(SI_CYRHUD_APRIL1_TOOLTIP),
            getFunc = function() return CyrHUD.cfg.aprOff or false end,
            setFunc = function(v) CyrHUD.cfg.aprOff = v; CyrHUD:refresh() end,
        })
    end]]
end

function CyrHUD.playerInit()
    local self = CyrHUD

    if not self.initLAM then
        self.addonInit()
    end

    if IsPlayerInAvAWorld() then
 		if IsInImperialCity() and not self.cfg.enableInIC then
		   self:deinit()
		   return
		elseif not IsInImperialCity() and not self.cfg.enableInCyro then
		   self:deinit()
		   return
		end
		
		if self.visible then
            if CyrHUD.campaignID ~= GetCurrentCampaignId() then self:refresh() end 
        else
            self:init()
        end
	    CyrHUD.campaignID = GetCurrentCampaignId()	
    elseif self.visible then
        self:deinit()
    end
end

EVENT_MANAGER:RegisterForEvent(CyrHUD.addonVars.name .. "-init", EVENT_PLAYER_ACTIVATED, CyrHUD.playerInit)
