-- This file is part of CyrHUD
--
-- (C) 2014 Scott Yeskie (Sasky)
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.

CyrHUD = CyrHUD or {}

local function n0(val) if val == nil then return 0 end return val end

-- Setup class
CyrHUD.MovingObjective = {}
CyrHUD.MovingObjective.__index = CyrHUD.MovingObjective
CyrHUD.MovingObjective.type = "MovingObjective"

setmetatable(CyrHUD.MovingObjective, {
    __call = function (cls, ...)
        return cls.new(...)
    end,
})


local shortenObjName = function(str)
    return str:gsub(",..$", ""):gsub("%^.d$", "")
	--EN
	    :gsub("Elder Scroll of ", "")
end

----------------------------------------------
-- Creation
----------------------------------------------

CyrHUD.MovingObjective.new = function(objectiveId)
    local self = setmetatable({}, CyrHUD.MovingObjective)

    self.startMovingObjective = self.startMovingObjective or GetTimeStamp()
    self.endMovingObjective = nil
    self.objectiveId = objectiveId


    --self:update()

    return self
end

----------------------------------------------
-- Label update
----------------------------------------------

--Label fields
local L_ICON, L_UA = "img1", "img2"
local CYROCHAT_ICON = 'img4'
local L_NAME, L_ATT_SIEGE, L_DEF_SIEGE, L_TIME = "txt1", "txt2", "txt3", "txt4"
local L_LUMB, L_MINE, L_FARM = "img5", "img6", "img7"
local L_SCROLL = "img8"
local L_ICON_DEATHS_AD, L_ICON_DEATHS_EP, L_ICON_DEATHS_DC = "img12", "img13", "img14"
local L_DEATHS_AD, L_DEATHS_EP, L_DEATHS_DC = "txt11", "txt12", "txt13"
local L_ATT_SIEGE_ICON, L_DEF_SIEGE_ICON = "img15", "img16"
local L_CONNECTED = "img17"
local L_ARROW = "img18"
local L_HOLDER = "txt16"

function CyrHUD.MovingObjective:configureLabel(label)

    local obj = CyrHUD.MovingObjective[self.objectiveId]
	
    label:exposeControls(2,4)
    


    -- Objective icon
    label:getControl(L_ICON):SetDrawLayer(2)
	
	if obj.objectiveType == OBJECTIVE_ARTIFACT_DEFENSIVE or obj.objectiveType == OBJECTIVE_ARTIFACT_OFFENSIVE then
	    label:positionControl(L_ICON, 64, 64, -14, -12) -- 40 40 -2 -2
	else
	    label:positionControl(L_ICON, 40, 40, -2, -2)
	end
	
	

    --Under attack graphic
    label:positionControl(L_UA, 40, 40, -2, -2)
    local ua = label:getControl(L_UA)
    ua:SetDrawLayer(1)
    ua:SetTexture(CyrHUD.info.underAttack)

    --Objective name
    label:positionControl(L_NAME, 150, 30, 35, 5)
	
	--Holder name
	label:positionControl(L_HOLDER, 150, 30, 35, 23)

    --Time
    label:positionControl(L_TIME, 35, 30, 245, 5)
end

function CyrHUD.SetArtifactStateData(_, artifactName, keepId, characterName, playerAlliance, objectiveControlEvent, objectiveControlState, campaignId, displayName)
   if campaignId == 0 then return end
   
   CyrHUD.ArtifactHolders = CyrHUD.ArtifactHolders or {}
   
   if objectiveControlEvent == OBJECTIVE_CONTROL_EVENT_FLAG_TAKEN then
       CyrHUD.ArtifactHolders[artifactName] = displayName 
   
   elseif objectiveControlEvent == OBJECTIVE_CONTROL_EVENT_FLAG_DROPPED then
       CyrHUD.ArtifactHolders[artifactName] = ""
   end

end

function CyrHUD.MovingObjective:updateLabel(label)

    label:getControl(L_ICON_DEATHS_AD):SetHidden(true)
	label:getControl(L_ICON_DEATHS_EP):SetHidden(true)
	label:getControl(L_ICON_DEATHS_DC):SetHidden(true)
    label:getControl(L_DEATHS_AD):SetHidden(true)
	label:getControl(L_DEATHS_EP):SetHidden(true)
	label:getControl(L_DEATHS_DC):SetHidden(true)
    label:getControl(L_ATT_SIEGE_ICON):SetHidden(true)
	label:getControl(L_DEF_SIEGE_ICON):SetHidden(true)
	label:getControl(L_CONNECTED):SetHidden(true)
    label:getControl(L_ARROW):SetAlpha(0)

    local obj = CyrHUD.MovingObjective[self.objectiveId]
	local underAttack = obj.objectiveControlEvent == OBJECTIVE_CONTROL_EVENT_UNDER_ATTACK
	
	--objective icon
    label:getControl(L_ICON):SetTexture(obj.texture)
	label:getControl(L_ICON):SetColor(ZO_ColorDef:New(1,1,1,1):UnpackRGBA())
	
	if obj.objectiveType == OBJECTIVE_ARTIFACT_DEFENSIVE or obj.objectiveType == OBJECTIVE_ARTIFACT_OFFENSIVE then
	    label:positionControl(L_ICON, 64, 64, -14, -12) -- 40 40 -2 -2
	else
	    label:positionControl(L_ICON, 40, 40, -2, -2)
	end
	
    label:getControl(L_UA):SetHidden(not underAttack)
	label:getControl(L_LUMB):SetHidden(true)
	label:getControl(L_MINE):SetHidden(true)
	label:getControl(L_FARM):SetHidden(true)
    label:getControl(L_SCROLL):SetHidden(true)
	label:getControl(L_ATT_SIEGE):SetHidden(true)
	label:getControl(L_DEF_SIEGE):SetHidden(true)
	label:getControl(CYROCHAT_ICON):SetHidden(true)
	
	label:getControl(L_HOLDER):SetHidden(false)
	

    --objective name
    local name = label:getControl(L_NAME)
    name:SetText(shortenObjName(obj.name))
    name:SetColor(CyrHUD.info[obj.holdingAlliance].color:UnpackRGBA())

    --objective holder name
    local name = label:getControl(L_HOLDER)
	
	local holderName
	if CyrHUD.ArtifactHolders and CyrHUD.ArtifactHolders[obj.name] then
	    holderName = CyrHUD.ArtifactHolders[obj.name]
	else
	    _, holderName = GetCarryableObjectiveHoldingCharacterInfo(obj.keepId, obj.objectiveId, CyrHUD.battleContext)
		if holderName == nil or holderName == "" then
		   _, holderName = GetCarryableObjectiveLastHoldingCharacterInfo(obj.keepId, obj.objectiveId, CyrHUD.battleContext) 
		end
	end
	
	if obj.holdingAlliance == 0 then
	    holderName = ""
	end
	
    name:SetText(holderName)
    name:SetColor(CyrHUD.colors.white:UnpackRGBA())	

    --Time
    label:getControl(L_TIME):SetText(self:getDuration())
	if self.endMovingObjective then 
        label:getControl(L_TIME):SetColor(CyrHUD.info[self.holdingAlliance].color:UnpackRGBA())
	else
        label:getControl(L_TIME):SetColor(CyrHUD.info[ALLIANCE_NONE].color:UnpackRGBA())	
	end 
	
	
	
	label:getControl(L_TIME):SetColor(CyrHUD.info[ALLIANCE_NONE].color:UnpackRGBA())

    --Background color
    label.main:SetCenterColor(self:getBGColor():UnpackRGBA())
end

----------------------------------------------
-- Model update
----------------------------------------------

function CyrHUD.MovingObjective:update()
    self.holdingAlliance, self.lastHoldingAlliance = GetCarryableObjectiveHoldingAllianceInfo(self.keepId, self.objectiveId, CyrHUD.battleContext)   
   
        if self.endMovingObjective then
            --Remove after time
            if GetDiffBetweenTimeStamps(GetTimeStamp(), self.endMovingObjective) > 15 then
                CyrHUD.MovingObjective[self.objectiveId] = nil
            end
        end

end

function CyrHUD.MovingObjective:restart()
    self.endMovingObjective = nil
end


----------------------------------------------
-- Getters
----------------------------------------------

--[[
    @return red, green, blue, alpha for background color
        all in range [0,1]
--]]
function CyrHUD.MovingObjective:getBGColor()
    if self.endMovingObjective then
        if self.holdingAlliance == GetUnitAlliance("player") then
            return CyrHUD.info.defendedColor
        end

        return CyrHUD.info.endAttackColor
    end

    local delta = GetDiffBetweenTimeStamps(GetTimeStamp(), self.startMovingObjective)

    if delta < 60 then
        CyrHUD.info.newAttackColor:Lerp(CyrHUD.info.defaultBGColor, delta/120)
    end

    return CyrHUD.info.defaultBGColor
end

--[[
    @return elapsed time of battle event
        if an end is specified, will show total lenth
        otherwise will show current length
--]]
function CyrHUD.MovingObjective:getDuration()
    return CyrHUD.formatTime(GetDiffBetweenTimeStamps(self.endMovingObjective or GetTimeStamp(), self.startMovingObjective), false, true)
end
